#!/bin/sh

###variable###
UBOOT_DEV="/dev/mmcblk0p1"
LOG_PATH="/etc/logs"
LOG_FILE="${LOG_PATH}/update_uboot.log"
[ -d "${LOG_PATH}" ] || mkdir -p "${LOG_PATH}"

###function###
log()
{
  echo "$(date '+%F %T') $@" | tee -a $LOG_FILE
}

###main###
main()
{
    log "=============================="
    log "TS-X33 U-Boot Firmware Update"
    log "=============================="

    if [ ! -b "${UBOOT_DEV}" ]; then
        log "ERROR: can't find uboot device \"${UBOOT_DEV}\", exit."
        return -1
    fi

    cur_pcbinfo=`cat /proc/cmdline 2>/dev/null | tr " " "\n\r" | grep qnap_model | cut -d"=" -f 2`
    cur_version=`cat /proc/cmdline 2>/dev/null | tr " " "\n\r" | grep uboot_build_date | cut -d"=" -f 2`
    log "cur_pcbinfo=\"${cur_pcbinfo}\""
    log "cur_version=\"${cur_version}\""
    if [ "x${cur_pcbinfo}" = "x" ] || [ "x${cur_version}" = "x" ]; then
        model_name=$(getcfg "System Enclosure" "MODEL" -f "/etc/model.conf")
        if [ "x${model_name}" = "xTS-133" ]; then
            log "no pcbinfo, read from model.conf, model_name = ${model_name}"
            cur_pcbinfo="ts133"
        elif [ "x${model_name}" = "xTS-233" ]; then
            log "no pcbinfo, read from model.conf, model_name = ${model_name}"
            cur_pcbinfo="ts233"
        else
            log "ERROR: can't get current uboot version information, exit."
            return -1
        fi
        cur_version="000000000000"
        log "cur_pcbinfo=\"${cur_pcbinfo}\""
        log "cur_version=\"${cur_version}\""
    fi

    new_version=$(getcfg "UBOOT_VER" "${cur_pcbinfo}" -f update_uboot.conf | awk '{print $1}')
    log "new_version=\"${new_version}\""
    if [ "x${new_version}" = "x" ]; then
        log "ERROR: no available uboot firmware for pcbinfo=${cur_pcbinfo}, exit."
        return -1
    fi

    if [ ${new_version} -le ${cur_version} ]; then
        log "current uboot firmware is the latest version, exit."
        return 0
    fi

    if [ ! -f "${cur_pcbinfo}.uboot" ]; then
        log "ERROR: File \"${cur_pcbinfo}.uboot\" not exist, exit."
        return -1
    fi

    if [ ! -f "${cur_pcbinfo}.uboot.cksum" ]; then
        log "ERROR: File \"${cur_pcbinfo}.uboot.cksum\" not exist, exit."
        return -1
    fi

    cksum_1=$(cksum "${cur_pcbinfo}.uboot" 2>/dev/null | awk '{print $1}')
    cksum_2=$(cat "${cur_pcbinfo}.uboot.cksum" 2>/dev/null | awk '{print $1}')
    log "cksum_1=\"${cksum_1}\""
    log "cksum_2=\"${cksum_2}\""
    if [ "x${cksum_1}" = "x" ] || [ "x${cksum_2}" = "x" ] || [ "x${cksum_1}" != "x${cksum_2}" ]; then
        log "ERROR: checksum wrong, \"${cur_pcbinfo}.uboot\" may corrupted, exit."
        return -1
    fi

    log "write to uboot device \"${UBOOT_DEV}\", wait..."
    error_msg=$(dd if=${cur_pcbinfo}.uboot of=${UBOOT_DEV} bs=1M 2>&1)
    if [ $? != 0 ]; then
        log "ERROR: write failed."
        log "error_msg=\"${error_msg}\"."
        log "RETRY: write to uboot device \"${UBOOT_DEV}\", wait..."
        error_msg=$(dd if=${cur_pcbinfo}.uboot of=${UBOOT_DEV} bs=1M 2>&1)
        if [ $? != 0 ]; then
            log "ERROR: write failed."
            log "error_msg=\"${error_msg}\"."
            return -1
        fi
    fi

    sync
    rm -f *.uboot*

    log "write ok, read from uboot device, wait..."
    error_msg=$(dd if=${UBOOT_DEV} of=read.uboot bs=1M count=4 2>&1)
    if [ $? != 0 ]; then
        log "ERROR: read uboot device failed."
        log "error_msg=\"${error_msg}\"."
        return -1
    fi

    cksum_3=$(cksum "read.uboot" 2>/dev/null | awk '{print $1}')
    log "cksum_3=\"${cksum_3}\""
    if [ "x${cksum_2}" = "x" ] || [ "x${cksum_3}" = "x" ] || [ "x${cksum_2}" != "x${cksum_3}" ]; then
        log "ERROR: checksum wrong, uboot may corrupted, exit."
        return -1
    fi

    rm -f *.uboot*
    sync
    sleep 1
    log "U-Boot Firmware Update: PASS"
    echo "---- update u-boot firmware from \"${cur_version}\" to \"${new_version}\"." >/dev/kmsg
    return 0
}

###start###
main
